import React, { Component, ChangeEvent, MouseEvent } from 'react';
import './TodoApp.css';


// Represents one item in the todo list.
type TodoItem = {name: string; completed: boolean;};

type TodoProps = {};
type TodoState = {
    items: Array<TodoItem>,
    newName: string,
  };

export class TodoApp extends Component<TodoProps, TodoState> {
  constructor(props: TodoProps) {
    super(props);

    this.state = {items: [], newName: ""};
  }

  render = (): JSX.Element => {
    if (this.state.items.length === 0) {
      return (
        <div>
          <h2>To-Do List</h2>
          <p><i>No items</i> in the list.</p>
          <p>New item:
            <input type="text" value={this.state.newName}
                onChange={this.doNewNameChange} />
            <button type="button"
                onClick={this.doAddClick}>Add</button>
          </p>
        </div>);

    } else {
      return (
        <div>
          <h2>To-Do List</h2>
          {this.renderItems()}
          <p>Check the item to mark it completed. Check it again to remove it.</p>
          <p>New item:
            <input type="text" value={this.state.newName}
                onChange={this.doNewNameChange} />
            <button type="button"
                onClick={this.doAddClick}>Add</button>
          </p>
        </div>);
    }
  };

  renderItems = (): Array<JSX.Element> => {
    const items: Array<JSX.Element> = [];
    for (let i = 0; i < this.state.items.length; i++) {
      const item = this.state.items[i];
      if (item.completed) {
        items.push(
          <div key={i}>
            <input type="checkbox" id={"check" + i} checked={true}
                onChange={evt => this.doItemRemove(evt, i)}/>
            <label htmlFor={"check" + i}><s>{item.name}</s></label>
          </div>);
      } else {
        items.push(
          <div key={i}>
            <input type="checkbox" id={"check" + i} checked={false}
                onChange={evt => this.doItemCompleted(evt, i)} />
            <label htmlFor={"check" + i}>{item.name}</label>
          </div>);
      }
    }
    return items;
  };

  doNewNameChange = (evt: ChangeEvent<HTMLInputElement>) => {
    this.setState({newName: evt.target.value});
  };

  doAddClick = (_evt: MouseEvent<HTMLButtonElement>) => {  // _evt since unused
    // Ignore the request if the user hasn't entered a name.
    const name = this.state.newName.trim();
    if (name.length == 0)
      return;

    const items = this.state.items.concat([{name: name, completed: false}]);
    this.setState({items: items, newName: ""});
  };

  doItemCompleted = (_evt: ChangeEvent<HTMLInputElement>, index: number) => {
    const item = this.state.items[index];

    const items = this.state.items.slice(0, index)
        .concat([{name: item.name, completed: true}])
        .concat(this.state.items.slice(index + 1));
    this.setState({items: items});
  };

  doItemRemove = (_evt: ChangeEvent<HTMLInputElement>, index: number) => {
    const items = this.state.items.slice(0, index)
        .concat(this.state.items.slice(index + 1));
    this.setState({items: items});
  }
}
